package com.mortals.xhx.module.certificate.service.impl;

import com.mortals.xhx.common.utils.StringUtils;
import com.mortals.xhx.module.certificate.model.CertificateCatalogEntity;
import com.mortals.xhx.module.certificate.model.CertificateIndustryQuery;
import com.mortals.xhx.module.certificate.model.CertificateIndustryTreeSelect;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.xhx.module.certificate.dao.CertificateIndustryDao;
import com.mortals.xhx.module.certificate.model.CertificateIndustryEntity;
import com.mortals.xhx.module.certificate.service.CertificateIndustryService;
import org.springframework.util.ObjectUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * CertificateIndustryService
 * 行业目录 service实现
 *
 * @author zxfei
 * @date 2022-10-14
 */
@Service("certificateIndustryService")
public class CertificateIndustryServiceImpl extends AbstractCRUDServiceImpl<CertificateIndustryDao, CertificateIndustryEntity, Long> implements CertificateIndustryService {

    @Override
    protected void validData(CertificateIndustryEntity entity, Context context) throws AppException {
        if (StringUtils.isEmpty(entity.getIndustryName())) {
            throw new AppException("行业名称不能为空");
        }
        Map<String, Object> condition = new HashMap<>();
        condition.put("industryName", entity.getIndustryName());
        List<CertificateIndustryEntity> list = this.dao.getList(condition);
        if (CollectionUtils.isNotEmpty(list)) {
            throw new AppException("行业名称重复");
        }
    }

    @Override
    protected void saveBefore(CertificateIndustryEntity entity, Context context) throws AppException {
        super.saveBefore(entity, context);
        if (entity.getParentId() == null) {
            entity.setParentId(-1l);
        }
        CertificateIndustryEntity parentCertificateIndustryEntity = this.get(entity.getParentId());
        if (!ObjectUtils.isEmpty(parentCertificateIndustryEntity)) {
            entity.setAncestors(parentCertificateIndustryEntity.getAncestors() + "," + entity.getParentId());
        }
    }


    @Override
    protected void updateBefore(CertificateIndustryEntity entity, Context context) throws AppException {
        CertificateIndustryEntity newParentEntity = this.get(entity.getParentId());
        CertificateIndustryEntity oldEntity = this.get(entity.getId());
        if (!ObjectUtils.isEmpty(newParentEntity) && !ObjectUtils.isEmpty(oldEntity)) {
            String newAncestors = newParentEntity.getAncestors() + "," + newParentEntity.getId();
            String oldAncestors = oldEntity.getAncestors();
            entity.setAncestors(newAncestors);
            updateCertificateIndustryChildren(entity.getId(), newAncestors, oldAncestors, context);
        }
        super.updateBefore(entity, context);
    }


    /**
     * 修改子元素关系
     *
     * @param certificateIndustryId 被修改的行业目录ID
     * @param newAncestors          新的父ID集合
     * @param oldAncestors          旧的父ID集合
     */
    public void updateCertificateIndustryChildren(Long certificateIndustryId, String newAncestors, String oldAncestors, Context context) {
        List<CertificateIndustryEntity> children = getDao().selectChildrenCertificateIndustryById(certificateIndustryId.toString());
        for (CertificateIndustryEntity child : children) {
            child.setAncestors(child.getAncestors().replace(oldAncestors, newAncestors));
        }
        if (children.size() > 0) {
            this.updateAfter(children, context);
        }
    }


    @Override
    protected void removeBefore(Long[] ids, Context context) throws AppException {
        //有子节点 禁止删除
        if (hasChildByCertificateIndustryId(ids[0])) {
            throw new AppException("存在下级行业目录,不允许删除");
        }
        super.removeBefore(ids, context);
    }

    @Override
    public boolean hasChildByCertificateIndustryId(Long certificateIndustryId) {
        List<CertificateIndustryEntity> list = this.find(new CertificateIndustryQuery().parentId(certificateIndustryId));
        return list.size() > 0 ? true : false;
    }


    @Override
    public List<CertificateIndustryTreeSelect> buildCertificateIndustryTreeSelect(List<CertificateIndustryEntity> list) {
        List<CertificateIndustryEntity> returnList = new ArrayList<>();
        List<Long> tempList = list.stream().map(CertificateIndustryEntity::getId).collect(Collectors.toList());

        Iterator<CertificateIndustryEntity> iterator = list.iterator();
        while (iterator.hasNext()) {
            CertificateIndustryEntity certificateIndustryEntity = iterator.next();
            if (!tempList.contains(certificateIndustryEntity.getParentId())) {
                recursionFn(list, certificateIndustryEntity);
                returnList.add(certificateIndustryEntity);
            }
        }
        if (returnList.isEmpty()) {
            returnList = list;
        }
        return returnList.stream().map(CertificateIndustryTreeSelect::new).collect(Collectors.toList());
    }


    /**
     * 递归列表
     */
    private void recursionFn(List<CertificateIndustryEntity> list, CertificateIndustryEntity t) {
        // 得到子节点列表
        List<CertificateIndustryEntity> childList = getChildList(list, t);
        t.setChildren(childList);
        for (CertificateIndustryEntity tChild : childList) {
            if (hasChild(list, tChild)) {
                recursionFn(list, tChild);
            }
        }
    }

    /**
     * 判断是否有子节点
     */
    private boolean hasChild(List<CertificateIndustryEntity> list, CertificateIndustryEntity t) {
        return getChildList(list, t).size() > 0 ? true : false;
    }

    /**
     * 得到子节点列表
     */
    private List<CertificateIndustryEntity> getChildList(List<CertificateIndustryEntity> list, CertificateIndustryEntity t) {
        return list.stream().map(item -> {
            if (!ObjectUtils.isEmpty(item.getParentId()) && item.getParentId() == t.getId()) {
                return item;
            }
            return null;
        }).filter(f -> f != null).collect(Collectors.toList());
    }

    @Override
    public List<CertificateIndustryTreeSelect> getListByParentId(Long parentId, Context context) {
        if (ObjectUtils.isEmpty(parentId)) {
            parentId = -1L;
        }
        List<CertificateIndustryTreeSelect> collect = this.find(new CertificateIndustryQuery().parentId(parentId), context)
                .stream().map(item -> new CertificateIndustryTreeSelect(item))
                .collect(Collectors.toList());
        return collect;
    }

    @Override
    public List<CertificateIndustryTreeSelect> getAllListByParentId(Long parentId, Context context) {
        List<CertificateIndustryEntity> returnList = new ArrayList<>();
       //获取子树列表
        List<CertificateIndustryEntity> childsList = this.getDao().selectChildrenCertificateIndustryById(parentId.toString());

        CertificateIndustryEntity certificateIndustryEntity = this.get(parentId, context);
        childsList.add(certificateIndustryEntity);

        recursionFn(childsList, certificateIndustryEntity);
        returnList.add(certificateIndustryEntity);

/*        List<CertificateIndustryEntity> list = this.find(new CertificateIndustryQuery());
        List<CertificateIndustryEntity> returnList = new ArrayList<>();
        List<Long> tempList = list.stream().map(CertificateIndustryEntity::getId).collect(Collectors.toList());

        if (!tempList.contains(parentId)) {
            CertificateIndustryEntity certificateIndustryEntity = this.get(parentId, context);
            recursionFn(list, certificateIndustryEntity);
            returnList.add(certificateIndustryEntity);
        }*/
        return returnList.stream().map(CertificateIndustryTreeSelect::new).collect(Collectors.toList());
    }


}